@@ -1,7 +1,7 @@
 /* taname.c */
 
 /*
- * Copyright (C) 2012-2013 by Werner Lemberg.
+ * Copyright (C) 2012-2015 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -200,18 +200,24 @@ parse_name_records(FT_Byte** curp,
 
     if (font->info)
     {
-      if (font->info(r->platform_id,
-                     r->encoding_id,
-                     r->language_id,
-                     r->name_id,
-                     &r->len,
-                     &r->str,
-                     font->info_data))
-        continue;
+      /* we ignore the return value of `font->info' */
+      font->info(r->platform_id,
+                 r->encoding_id,
+                 r->language_id,
+                 r->name_id,
+                 &r->len,
+                 &r->str,
+                 font->info_data);
     }
+
     count++;
   }
 
+  /* let the user modify `name' table entries */
+  if (font->info && font->info_post)
+    /* we ignore the return value of `font->info_post' */
+    font->info_post(font->info_data);
+
   /* shrink name record array if necessary */
   n->name_records = (Name_Record*)realloc(n->name_records,
                                           count * sizeof (Name_Record));
